@@ -16,16 +16,20 @@
 
 package org.gradle.api.plugins.quality;
 
+import org.gradle.api.reporting.CustomizableHtmlReport;
 import org.gradle.api.reporting.ReportContainer;
 import org.gradle.api.reporting.SingleFileReport;
-import org.gradle.api.tasks.Nested;
+import org.gradle.api.tasks.Internal;
 
 /**
  * The reporting configuration for the {@link FindBugs} task.
  *
  * Only one of the reports can be enabled when the task executes. If more than one is enabled, an {@link org.gradle.api.InvalidUserDataException}
  * will be thrown.
+ *
+ * @deprecated FindBugs is unmaintained and does not support bytecode compiled for Java 9 and above.
  */
+@Deprecated
 public interface FindBugsReports extends ReportContainer<SingleFileReport> {
 
     /**
@@ -33,7 +37,7 @@ public interface FindBugsReports extends ReportContainer<SingleFileReport> {
      *
      * @return The findbugs XML report
      */
-    @Nested
+    @Internal
     FindBugsXmlReport getXml();
 
     /**
@@ -41,15 +45,15 @@ public interface FindBugsReports extends ReportContainer<SingleFileReport> {
      *
      * @return The findbugs HTML report
      */
-    @Nested
-    SingleFileReport getHtml();
+    @Internal
+    CustomizableHtmlReport getHtml();
 
     /**
      * The findbugs Text report
      *
      * @return The findbugs Text report
      */
-    @Nested
+    @Internal
     SingleFileReport getText();
 
     /**
@@ -57,6 +61,6 @@ public interface FindBugsReports extends ReportContainer<SingleFileReport> {
      *
      * @return The findbugs Emacs report
      */
-    @Nested
+    @Internal
     SingleFileReport getEmacs();
 }
