@@ -26,17 +26,17 @@ import org.gradle.api.internal.BuildDefinition;
 import org.gradle.api.internal.SettingsInternal;
 import org.gradle.api.internal.artifacts.DefaultBuildIdentifier;
 import org.gradle.api.internal.project.ProjectStateRegistry;
-import org.gradle.initialization.BuildRequestContext;
 import org.gradle.initialization.GradleLauncherFactory;
 import org.gradle.internal.build.BuildState;
 import org.gradle.internal.build.BuildStateRegistry;
 import org.gradle.internal.build.IncludedBuildState;
+import org.gradle.internal.build.NestedRootBuild;
 import org.gradle.internal.build.RootBuildState;
 import org.gradle.internal.build.StandAloneNestedBuild;
 import org.gradle.internal.concurrent.CompositeStoppable;
 import org.gradle.internal.concurrent.Stoppable;
 import org.gradle.internal.event.ListenerManager;
-import org.gradle.internal.service.ServiceRegistry;
+import org.gradle.internal.service.scopes.BuildTreeScopeServices;
 import org.gradle.internal.text.TreeFormatter;
 import org.gradle.util.Path;
 
@@ -52,7 +52,7 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
     private final IncludedBuildDependencySubstitutionsBuilder dependencySubstitutionsBuilder;
     private final GradleLauncherFactory gradleLauncherFactory;
     private final ListenerManager listenerManager;
-    private final ServiceRegistry rootServices;
+    private final BuildTreeScopeServices rootServices;
 
     // TODO: Locking around this state
     private RootBuildState rootBuild;
@@ -60,7 +60,7 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
     private final Map<File, IncludedBuildState> includedBuilds = Maps.newLinkedHashMap();
     private final List<IncludedBuildState> pendingIncludedBuilds = Lists.newArrayList();
 
-    public DefaultIncludedBuildRegistry(IncludedBuildFactory includedBuildFactory, ProjectStateRegistry projectRegistry, IncludedBuildDependencySubstitutionsBuilder dependencySubstitutionsBuilder, GradleLauncherFactory gradleLauncherFactory, ListenerManager listenerManager, ServiceRegistry rootServices) {
+    public DefaultIncludedBuildRegistry(IncludedBuildFactory includedBuildFactory, ProjectStateRegistry projectRegistry, IncludedBuildDependencySubstitutionsBuilder dependencySubstitutionsBuilder, GradleLauncherFactory gradleLauncherFactory, ListenerManager listenerManager, BuildTreeScopeServices rootServices) {
         this.includedBuildFactory = includedBuildFactory;
         this.projectRegistry = projectRegistry;
         this.dependencySubstitutionsBuilder = dependencySubstitutionsBuilder;
@@ -78,15 +78,24 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
     }
 
     @Override
-    public RootBuildState addRootBuild(BuildDefinition buildDefinition, BuildRequestContext requestContext) {
+    public RootBuildState createRootBuild(BuildDefinition buildDefinition) {
         if (rootBuild != null) {
             throw new IllegalStateException("Root build already defined.");
         }
-        rootBuild = new DefaultRootBuildState(buildDefinition, requestContext, gradleLauncherFactory, listenerManager, rootServices);
+        rootBuild = new DefaultRootBuildState(buildDefinition, gradleLauncherFactory, listenerManager, rootServices);
         addBuild(rootBuild);
         return rootBuild;
     }
 
+    @Override
+    public void attachRootBuild(RootBuildState rootBuild) {
+        if (this.rootBuild != null) {
+            throw new IllegalStateException("Root build already defined.");
+        }
+        this.rootBuild = rootBuild;
+        addBuild(rootBuild);
+    }
+
     private void addBuild(BuildState build) {
         BuildState before = builds.put(build.getBuildIdentifier(), build);
         if (before != null) {
@@ -126,44 +135,18 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
         return buildState;
     }
 
-    @Override
-    public void register(BuildState build) {
-        addBuild(build);
-        if (build instanceof RootBuildState) {
-            if (rootBuild != null) {
-                throw new IllegalStateException("Root build already defined.");
-            }
-            rootBuild = (RootBuildState) build;
-        }
-    }
-
-    @Override
-    public void registerRootBuild(SettingsInternal settings) {
-        validateIncludedBuilds(settings);
-
-        projectRegistry.registerProjects(rootBuild);
-
-        Collection<IncludedBuildState> includedBuilds = getIncludedBuilds();
-        registerProjects(includedBuilds);
-    }
-
     @Override
     public void beforeConfigureRootBuild() {
         registerSubstitutions(includedBuilds.values());
     }
 
-    private void registerProjects(Collection<IncludedBuildState> includedBuilds) {
-        for (IncludedBuildState includedBuild : includedBuilds) {
-            projectRegistry.registerProjects(includedBuild);
-        }
-    }
-
-    private void validateIncludedBuilds(SettingsInternal settings) {
+    @Override
+    public void finalizeIncludedBuilds() {
+        SettingsInternal settings = getRootBuild().getLoadedSettings();
         SetMultimap<String, IncludedBuildState> names = LinkedHashMultimap.create();
         while (!pendingIncludedBuilds.isEmpty()) {
             IncludedBuildState build = pendingIncludedBuilds.remove(0);
-            // This implicitly loads the settings, possibly discovering more included builds
-            // Should make this an explicit step instead
+            build.loadSettings();
             String buildName = build.getName();
             names.put(buildName, build);
             if (settings.getRootProject().getName().equals(buildName)) {
@@ -200,7 +183,6 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
         IncludedBuildState includedBuild = includedBuilds.get(buildDefinition.getBuildRootDir());
         if (includedBuild == null) {
             includedBuild = registerBuild(buildDefinition, true);
-            projectRegistry.registerProjects(includedBuild);
         }
         return includedBuild;
     }
@@ -212,18 +194,13 @@ public class DefaultIncludedBuildRegistry implements BuildStateRegistry, Stoppab
         }
         BuildIdentifier buildIdentifier = idFor(buildDefinition.getName());
         Path identityPath = pathFor(owner, buildIdentifier.getName());
-        DefaultNestedBuild build = new DefaultNestedBuild(buildIdentifier, identityPath, buildDefinition, owner, new BuildStateListener() {
-            @Override
-            public void projectsKnown(BuildState build1) {
-                projectRegistry.registerProjects(build1);
-            }
-        });
+        DefaultNestedBuild build = new DefaultNestedBuild(buildIdentifier, identityPath, buildDefinition, owner);
         addBuild(build);
         return build;
     }
 
     @Override
-    public StandAloneNestedBuild addNestedBuildTree(BuildDefinition buildDefinition, BuildState owner) {
+    public NestedRootBuild addNestedBuildTree(BuildDefinition buildDefinition, BuildState owner) {
         if (buildDefinition.getName() != null || buildDefinition.getBuildRootDir() != null) {
             throw new UnsupportedOperationException("Not yet implemented."); // but should be
         }
