@@ -16,8 +16,8 @@
 package org.gradle.api.artifacts.component;
 
 import org.gradle.api.Incubating;
+import org.gradle.api.artifacts.ModuleIdentifier;
 import org.gradle.api.artifacts.VersionConstraint;
-import org.gradle.api.attributes.AttributeContainer;
 import org.gradle.internal.scan.UsedByScanPlugin;
 
 /**
@@ -26,7 +26,6 @@ import org.gradle.internal.scan.UsedByScanPlugin;
  * @since 1.10
  */
 @UsedByScanPlugin
-@Incubating
 public interface ModuleComponentSelector extends ComponentSelector {
     /**
      * The group of the module to select the component from.
@@ -61,13 +60,13 @@ public interface ModuleComponentSelector extends ComponentSelector {
     VersionConstraint getVersionConstraint();
 
     /**
-     * The attributes of the module to select the component from. The attributes only include
-     * selector specific attributes. This means it typically doesn't include any consumer specific attribute.
+     * The module identifier of the component. Returns the same information
+     * as {@link #getGroup()} and {@link #getModule()}.
      *
-     * @return the attributes
+     * @return the module identifier
      *
-     * @since 4.8
+     * @since 4.9
      */
     @Incubating
-    AttributeContainer getAttributes();
+    ModuleIdentifier getModuleIdentifier();
 }
