@@ -17,19 +17,19 @@ package org.gradle.api.internal;
 
 import groovy.lang.Closure;
 import org.gradle.api.Action;
-import org.gradle.api.NamedDomainObjectProvider;
 import org.gradle.api.InvalidUserDataException;
 import org.gradle.api.Named;
 import org.gradle.api.NamedDomainObjectContainer;
+import org.gradle.api.NamedDomainObjectProvider;
 import org.gradle.api.Namer;
 import org.gradle.api.reflect.HasPublicType;
 import org.gradle.api.reflect.TypeOf;
 import org.gradle.internal.Actions;
 import org.gradle.internal.Cast;
-import org.gradle.internal.ImmutableActionSet;
 import org.gradle.internal.metaobject.ConfigureDelegate;
 import org.gradle.internal.reflect.Instantiator;
 import org.gradle.util.ConfigureUtil;
+import org.gradle.util.DeprecationLogger;
 
 import javax.annotation.Nullable;
 
@@ -38,12 +38,31 @@ import static org.gradle.api.reflect.TypeOf.typeOf;
 
 public abstract class AbstractNamedDomainObjectContainer<T> extends DefaultNamedDomainObjectSet<T> implements NamedDomainObjectContainer<T>, HasPublicType {
 
-    protected AbstractNamedDomainObjectContainer(Class<T> type, Instantiator instantiator, Namer<? super T> namer) {
-        super(type, instantiator, namer);
+    protected AbstractNamedDomainObjectContainer(Class<T> type, Instantiator instantiator, Namer<? super T> namer, CollectionCallbackActionDecorator callbackDecorator) {
+        super(type, instantiator, namer, callbackDecorator);
     }
 
+    protected AbstractNamedDomainObjectContainer(Class<T> type, Instantiator instantiator, CollectionCallbackActionDecorator callbackActionDecorator) {
+        super(type, instantiator, Named.Namer.forType(type), callbackActionDecorator);
+    }
+
+    /**
+     * This internal constructor is used by 'nebula.plugin-plugin' which we test as part of our ci pipeline.
+     * */
+    @Deprecated
+    protected AbstractNamedDomainObjectContainer(Class<T> type, Instantiator instantiator, Namer<? super T> namer) {
+        this(type, instantiator, namer, CollectionCallbackActionDecorator.NOOP);
+        DeprecationLogger.nagUserOfDeprecated("Internal API constructor AbstractNamedDomainObjectContainer(Class<T>, Instantiator, Namer<? extends T>)", "Don't use internal API");
+    }
+
+
+    /**
+     * This internal constructor is used by the 'com.eriwen.gradle.css' and 'com.eriwen.gradle.js' plugin which we test as part of our ci pipeline.
+     * */
+    @Deprecated
     protected AbstractNamedDomainObjectContainer(Class<T> type, Instantiator instantiator) {
-        super(type, instantiator, Named.Namer.forType(type));
+        this(type, instantiator, CollectionCallbackActionDecorator.NOOP);
+        DeprecationLogger.nagUserOfDeprecated("Internal API constructor FactoryNamedDomainObjectContaineronstructor AbstractNamedDomainObjectContainer(Class<T>, Instantiator)", "Don't use internal API");
     }
 
     /**
@@ -52,6 +71,7 @@ public abstract class AbstractNamedDomainObjectContainer<T> extends DefaultNamed
     protected abstract T doCreate(String name);
 
     public T create(String name) {
+        assertMutable("create(String)");
         return create(name, Actions.doNothing());
     }
 
@@ -64,10 +84,12 @@ public abstract class AbstractNamedDomainObjectContainer<T> extends DefaultNamed
     }
 
     public T create(String name, Closure configureClosure) {
+        assertMutable("create(String, Closure)");
         return create(name, ConfigureUtil.configureUsing(configureClosure));
     }
 
     public T create(String name, Action<? super T> configureAction) throws InvalidUserDataException {
+        assertMutable("create(String, Action)");
         assertCanAdd(name);
         T object = doCreate(name);
         add(object);
@@ -96,84 +118,34 @@ public abstract class AbstractNamedDomainObjectContainer<T> extends DefaultNamed
 
     @Override
     public NamedDomainObjectProvider<T> register(String name) throws InvalidUserDataException {
+        assertMutable("register(String)");
         return createDomainObjectProvider(name, null);
     }
 
     @Override
     public NamedDomainObjectProvider<T> register(String name, Action<? super T> configurationAction) throws InvalidUserDataException {
+        assertMutable("register(String, Action)");
         return createDomainObjectProvider(name, configurationAction);
     }
 
     protected NamedDomainObjectProvider<T> createDomainObjectProvider(String name, @Nullable Action<? super T> configurationAction) {
         assertCanAdd(name);
         NamedDomainObjectProvider<T> provider = Cast.uncheckedCast(
-            getInstantiator().newInstance(NamedDomainObjectCreatingProvider.class, AbstractNamedDomainObjectContainer.this, name, configurationAction)
+            getInstantiator().newInstance(NamedDomainObjectCreatingProvider.class, AbstractNamedDomainObjectContainer.this, name, getType(), configurationAction)
         );
         addLater(provider);
         return provider;
     }
 
     // Cannot be private due to reflective instantiation
-    public class NamedDomainObjectCreatingProvider<I extends T> extends AbstractNamedDomainObjectProvider<I> {
-        private I object;
-        private Throwable cause;
-        private ImmutableActionSet<I> onCreate;
-
-        public NamedDomainObjectCreatingProvider(String name, @Nullable Action<? super I> configureAction) {
-            super(name);
-            this.onCreate = ImmutableActionSet.<I>empty().mergeFrom(getEventRegister().getAddActions());
-
-            if (configureAction != null) {
-                configure(configureAction);
-            }
+    public class NamedDomainObjectCreatingProvider<I extends T> extends AbstractDomainObjectCreatingProvider<I> {
+        public NamedDomainObjectCreatingProvider(String name, Class<I> type, @Nullable Action<? super I> configureAction) {
+            super(name, type, configureAction);
         }
 
         @Override
-        public boolean isPresent() {
-            return findDomainObject(getName()) != null;
-        }
-
-        public void configure(final Action<? super I> action) {
-            if (object != null) {
-                // Already realized, just run the action now
-                action.execute(object);
-                return;
-            }
-            // Collect any container level add actions then add the object specific action
-            onCreate = onCreate.mergeFrom(getEventRegister().getAddActions()).add(action);
-        }
-
-        @Override
-        public I getOrNull() {
-            if (cause != null) {
-                throw createIllegalStateException();
-            }
-            if (object == null) {
-                object = getType().cast(findByNameWithoutRules(getName()));
-                if (object == null) {
-                    try {
-                        // Collect any container level add actions added since the last call to configure()
-                        onCreate = onCreate.mergeFrom(getEventRegister().getAddActions());
-
-                        // Create the domain object
-                        object = (I) doCreate(getName());
-
-                        // Register the domain object
-                        add(object, onCreate);
-                    } catch (RuntimeException ex) {
-                        cause = ex;
-                        throw createIllegalStateException();
-                    } finally {
-                        // Discard state that is no longer required
-                        onCreate = ImmutableActionSet.empty();
-                    }
-                }
-            }
-            return object;
-        }
-
-        private IllegalStateException createIllegalStateException() {
-            return new IllegalStateException(String.format("Could not create domain object '%s' (%s)", getName(), getType().getSimpleName()), cause);
+        protected I createDomainObject() {
+            return Cast.uncheckedCast(doCreate(getName()));
         }
     }
 }
