@@ -23,7 +23,7 @@ import com.google.common.collect.Sets;
 import org.gradle.BuildListener;
 import org.gradle.BuildResult;
 import org.gradle.api.Task;
-import org.gradle.api.internal.ExceptionAnalyser;
+import org.gradle.initialization.exception.ExceptionAnalyser;
 import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.internal.SettingsInternal;
 import org.gradle.composite.internal.IncludedBuildControllers;
@@ -31,7 +31,7 @@ import org.gradle.configuration.BuildConfigurer;
 import org.gradle.execution.BuildConfigurationActionExecuter;
 import org.gradle.execution.BuildExecuter;
 import org.gradle.execution.MultipleBuildFailures;
-import org.gradle.execution.TaskExecutionGraphInternal;
+import org.gradle.execution.taskgraph.TaskExecutionGraphInternal;
 import org.gradle.internal.build.PublicBuildPath;
 import org.gradle.internal.concurrent.CompositeStoppable;
 import org.gradle.internal.operations.BuildOperationCategory;
@@ -42,6 +42,7 @@ import org.gradle.internal.operations.RunnableBuildOperation;
 import org.gradle.internal.service.scopes.BuildScopeServices;
 import org.gradle.internal.taskgraph.CalculateTaskGraphBuildOperationType;
 
+import javax.annotation.Nullable;
 import java.util.ArrayList;
 import java.util.List;
 import java.util.Set;
@@ -54,7 +55,16 @@ public class DefaultGradleLauncher implements GradleLauncher {
     };
 
     private enum Stage {
-        Load, LoadBuild, Configure, TaskGraph, Build, Finished
+        LoadSettings, Configure, TaskGraph, RunTasks() {
+            @Override
+            String getDisplayName() {
+                return "Build";
+            }
+        }, Finished;
+
+        String getDisplayName() {
+            return name();
+        }
     }
 
     private static final LoadBuildBuildOperationType.Result RESULT = new LoadBuildBuildOperationType.Result() {
@@ -110,7 +120,7 @@ public class DefaultGradleLauncher implements GradleLauncher {
 
     @Override
     public SettingsInternal getLoadedSettings() {
-        doBuildStages(Stage.Load);
+        doBuildStages(Stage.LoadSettings);
         return settings;
     }
 
@@ -121,21 +131,21 @@ public class DefaultGradleLauncher implements GradleLauncher {
     }
 
     public GradleInternal executeTasks() {
-        doBuildStages(Stage.Build);
+        doBuildStages(Stage.RunTasks);
         return gradle;
     }
 
     @Override
     public void finishBuild() {
         if (stage != null) {
-            finishBuild(new BuildResult(stage.name(), gradle, null));
+            finishBuild(stage.getDisplayName(), null);
         }
     }
 
     private void doBuildStages(Stage upTo) {
         try {
             loadSettings();
-            if (upTo == Stage.Load) {
+            if (upTo == Stage.LoadSettings) {
                 return;
             }
             configureBuild();
@@ -147,24 +157,42 @@ public class DefaultGradleLauncher implements GradleLauncher {
                 return;
             }
             runTasks();
+            if (upTo == Stage.RunTasks) {
+                return;
+            }
             finishBuild();
         } catch (Throwable t) {
-            Throwable failure = exceptionAnalyser.transform(t);
-            finishBuild(new BuildResult(upTo.name(), gradle, failure));
-            throw new ReportedException(failure);
+            finishBuild(upTo.getDisplayName(), t);
         }
     }
 
-    private void finishBuild(BuildResult result) {
+    private void finishBuild(String action, @Nullable Throwable stageFailure) {
         if (stage == Stage.Finished) {
             return;
         }
 
-        includedBuildControllers.finishBuild();
-
-        buildListener.buildFinished(result);
-
+        RuntimeException reportableFailure = stageFailure == null ? null : exceptionAnalyser.transform(stageFailure);
+        BuildResult buildResult = new BuildResult(action, gradle, reportableFailure);
+        List<Throwable> failures = new ArrayList<Throwable>();
+        includedBuildControllers.finishBuild(failures);
+        try {
+            buildListener.buildFinished(buildResult);
+        } catch (Throwable t) {
+            failures.add(t);
+        }
         stage = Stage.Finished;
+
+        if (failures.isEmpty() && reportableFailure != null) {
+            throw reportableFailure;
+        }
+        if (!failures.isEmpty()) {
+            if (stageFailure instanceof MultipleBuildFailures) {
+                failures.addAll(0, ((MultipleBuildFailures) stageFailure).getCauses());
+            } else if (stageFailure != null) {
+                failures.add(0, stageFailure);
+            }
+            throw exceptionAnalyser.transform(new MultipleBuildFailures(failures));
+        }
     }
 
     private void loadSettings() {
@@ -173,12 +201,12 @@ public class DefaultGradleLauncher implements GradleLauncher {
 
             buildOperationExecutor.run(new LoadBuild());
 
-            stage = Stage.Load;
+            stage = Stage.LoadSettings;
         }
     }
 
     private void configureBuild() {
-        if (stage == Stage.Load) {
+        if (stage == Stage.LoadSettings) {
             buildOperationExecutor.run(new ConfigureBuild());
 
             stage = Stage.Configure;
@@ -218,7 +246,7 @@ public class DefaultGradleLauncher implements GradleLauncher {
 
         buildOperationExecutor.run(new ExecuteTasks());
 
-        stage = Stage.Build;
+        stage = Stage.RunTasks;
     }
 
     /**
@@ -253,17 +281,17 @@ public class DefaultGradleLauncher implements GradleLauncher {
         @Override
         public BuildOperationDescriptor.Builder description() {
             return BuildOperationDescriptor.displayName(gradle.contextualize("Load build"))
-                .details(new LoadBuildBuildOperationType.Details() {
-                    @Override
-                    public String getBuildPath() {
-                        return gradle.getIdentityPath().toString();
-                    }
+                    .details(new LoadBuildBuildOperationType.Details() {
+                        @Override
+                        public String getBuildPath() {
+                            return gradle.getIdentityPath().toString();
+                        }
 
-                    @Override
-                    public String getIncludedBy() {
-                        return fromBuild == null ? null : fromBuild.getBuildPath().toString();
-                    }
-                });
+                        @Override
+                        public String getIncludedBy() {
+                            return fromBuild == null ? null : fromBuild.getBuildPath().toString();
+                        }
+                    });
         }
     }
 
@@ -338,12 +366,12 @@ public class DefaultGradleLauncher implements GradleLauncher {
         @Override
         public BuildOperationDescriptor.Builder description() {
             return BuildOperationDescriptor.displayName(gradle.contextualize("Calculate task graph"))
-                .details(new CalculateTaskGraphBuildOperationType.Details() {
-                    @Override
-                    public String getBuildPath() {
-                        return getGradle().getIdentityPath().getPath();
-                    }
-                });
+                    .details(new CalculateTaskGraphBuildOperationType.Details() {
+                        @Override
+                        public String getBuildPath() {
+                            return getGradle().getIdentityPath().getPath();
+                        }
+                    });
         }
     }
 
@@ -363,9 +391,9 @@ public class DefaultGradleLauncher implements GradleLauncher {
         public BuildOperationDescriptor.Builder description() {
             BuildOperationDescriptor.Builder builder = BuildOperationDescriptor.displayName(gradle.contextualize("Run tasks"));
             if (gradle.getParent() == null) {
-                builder.operationType(BuildOperationCategory.RUN_TASKS_ROOT_BUILD);
+                builder.operationType(BuildOperationCategory.RUN_WORK_ROOT_BUILD);
             } else {
-                builder.operationType(BuildOperationCategory.RUN_TASKS);
+                builder.operationType(BuildOperationCategory.RUN_WORK);
             }
             builder.totalProgress(gradle.getTaskGraph().size());
             return builder;
@@ -383,12 +411,12 @@ public class DefaultGradleLauncher implements GradleLauncher {
         @Override
         public BuildOperationDescriptor.Builder description() {
             return BuildOperationDescriptor.displayName(gradle.contextualize("Notify projectsEvaluated listeners"))
-                .details(new NotifyProjectsEvaluatedBuildOperationType.Details() {
-                    @Override
-                    public String getBuildPath() {
-                        return gradle.getIdentityPath().toString();
-                    }
-                });
+                    .details(new NotifyProjectsEvaluatedBuildOperationType.Details() {
+                        @Override
+                        public String getBuildPath() {
+                            return gradle.getIdentityPath().toString();
+                        }
+                    });
         }
     }
 
