@@ -16,6 +16,8 @@
 
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact;
 
+import org.gradle.api.internal.tasks.TaskDependencyResolveContext;
+import org.gradle.internal.UncheckedException;
 import org.gradle.internal.operations.BuildOperationQueue;
 import org.gradle.internal.operations.RunnableBuildOperation;
 
@@ -27,8 +29,8 @@ public class BrokenResolvedArtifactSet implements ResolvedArtifactSet, ResolvedA
     }
 
     @Override
-    public void collectBuildDependencies(BuildDependenciesVisitor visitor) {
-        visitor.visitFailure(failure);
+    public void visitDependencies(TaskDependencyResolveContext context) {
+        context.visitFailure(failure);
     }
 
     @Override
@@ -36,6 +38,11 @@ public class BrokenResolvedArtifactSet implements ResolvedArtifactSet, ResolvedA
         return this;
     }
 
+    @Override
+    public void visitLocalArtifacts(LocalArtifactVisitor listener) {
+        throw UncheckedException.throwAsUncheckedException(failure);
+    }
+
     @Override
     public void visit(ArtifactVisitor visitor) {
         visitor.visitFailure(failure);
