@@ -16,26 +16,22 @@
 
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.builder;
 
-import com.google.common.base.Joiner;
-import com.google.common.collect.LinkedHashMultimap;
 import com.google.common.collect.Lists;
-import com.google.common.collect.Multimap;
 import org.gradle.api.Action;
-import org.gradle.api.Describable;
-import org.gradle.api.Transformer;
 import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
+import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
+import org.gradle.api.artifacts.result.ComponentSelectionReason;
 import org.gradle.api.attributes.AttributeContainer;
 import org.gradle.api.capabilities.Capability;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.RepositoryChainModuleSource;
-import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelector;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.ComponentResolutionState;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphComponent;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.conflicts.VersionConflictResolutionDetails;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.selectors.ResolvableSelectorState;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionDescriptorInternal;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionReasonInternal;
-import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.VersionSelectionReasons;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionReasons;
 import org.gradle.api.internal.attributes.ImmutableAttributes;
 import org.gradle.internal.Describables;
 import org.gradle.internal.DisplayName;
@@ -45,16 +41,12 @@ import org.gradle.internal.component.model.ComponentResolveMetadata;
 import org.gradle.internal.component.model.DefaultComponentOverrideMetadata;
 import org.gradle.internal.component.model.ModuleSource;
 import org.gradle.internal.resolve.ModuleVersionResolveException;
-import org.gradle.internal.resolve.RejectedByAttributesVersion;
-import org.gradle.internal.resolve.RejectedByRuleVersion;
-import org.gradle.internal.resolve.RejectedBySelectorVersion;
-import org.gradle.internal.resolve.RejectedVersion;
 import org.gradle.internal.resolve.resolver.ComponentMetaDataResolver;
 import org.gradle.internal.resolve.result.DefaultBuildableComponentResolveResult;
-import org.gradle.internal.text.TreeFormatter;
 
 import java.util.Collection;
 import java.util.List;
+import java.util.Set;
 
 /**
  * Resolution state for a given component
@@ -71,7 +63,6 @@ public class ComponentState implements ComponentResolutionState, DependencyGraph
     private final ModuleResolveState module;
     private final List<ComponentSelectionDescriptorInternal> selectionCauses = Lists.newArrayList();
     private final ImmutableCapability implicitCapability;
-    private Multimap<VersionSelector, String> rejectedBySelectors;
 
     private volatile ComponentResolveMetadata metadata;
 
@@ -196,7 +187,11 @@ public class ComponentState implements ComponentResolutionState, DependencyGraph
         ComponentOverrideMetadata componentOverrideMetadata = DefaultComponentOverrideMetadata.forDependency(firstSelectedBy.getDependencyMetadata());
 
         DefaultBuildableComponentResolveResult result = new DefaultBuildableComponentResolveResult();
+        if (tryResolveVirtualPlatform()) {
+            return;
+        }
         resolver.resolve(componentIdentifier, componentOverrideMetadata, result);
+
         if (result.getFailure() != null) {
             metadataResolveFailure = result.getFailure();
             return;
@@ -204,6 +199,22 @@ public class ComponentState implements ComponentResolutionState, DependencyGraph
         metadata = result.getMetadata();
     }
 
+    private boolean tryResolveVirtualPlatform() {
+        if (module.isVirtualPlatform()) {
+            for (ComponentState version : module.getAllVersions()) {
+                if (version != this) {
+                    ComponentResolveMetadata metadata = version.getMetadata();
+                    if (metadata instanceof LenientPlatformResolveMetadata) {
+                        LenientPlatformResolveMetadata lenient = (LenientPlatformResolveMetadata) metadata;
+                        this.metadata = lenient.withVersion((ModuleComponentIdentifier) componentIdentifier, id);
+                        return true;
+                    }
+                }
+            }
+        }
+        return false;
+    }
+
     public void setMetadata(ComponentResolveMetadata metaData) {
         this.metadata = metaData;
         this.metadataResolveFailure = null;
@@ -214,15 +225,14 @@ public class ComponentState implements ComponentResolutionState, DependencyGraph
     }
 
     @Override
-    public ComponentSelectionReasonInternal getSelectionReason() {
+    public ComponentSelectionReason getSelectionReason() {
         if (root) {
-            return VersionSelectionReasons.root();
+            return ComponentSelectionReasons.root();
         }
-        ComponentSelectionReasonInternal reason = VersionSelectionReasons.empty();
+        ComponentSelectionReasonInternal reason = ComponentSelectionReasons.empty();
         for (final SelectorState selectorState : module.getSelectors()) {
             if (selectorState.getFailure() == null) {
-                selectorState.addReasonsForSelector(reason, new RejectedBySelectorDescriptorBuilder(selectorState));
-
+                selectorState.addReasonsForSelector(reason);
             }
         }
         for (ComponentSelectionDescriptorInternal selectionCause : VersionConflictResolutionDetails.mergeCauses(selectionCauses)) {
@@ -232,8 +242,8 @@ public class ComponentState implements ComponentResolutionState, DependencyGraph
     }
 
     @Override
-    public void addCause(ComponentSelectionDescriptorInternal reason) {
-        selectionCauses.add(reason);
+    public void addCause(ComponentSelectionDescriptorInternal componentSelectionDescriptor) {
+        selectionCauses.add(componentSelectionDescriptor);
     }
 
     public void setRoot() {
@@ -327,32 +337,13 @@ public class ComponentState implements ComponentResolutionState, DependencyGraph
     }
 
     @Override
-    public void unmatched(Collection<RejectedBySelectorVersion> unmatchedVersions) {
-        for (RejectedBySelectorVersion unmatchedVersion : unmatchedVersions) {
-            registerRejections(unmatchedVersion, unmatchedVersion.getId().getVersion());
-        }
+    public Set<VirtualPlatformState> getPlatformOwners() {
+        return module.getPlatformOwners();
     }
 
     @Override
-    public void rejected(Collection<RejectedVersion> rejectedVersions) {
-        for (RejectedVersion rejectedVersion : rejectedVersions) {
-            String version = rejectedVersion.getId().getVersion();
-            if (rejectedVersion instanceof RejectedBySelectorVersion) {
-                registerRejections((RejectedBySelectorVersion) rejectedVersion, version);
-            } else if (rejectedVersion instanceof RejectedByRuleVersion) {
-                String reason = ((RejectedByRuleVersion) rejectedVersion).getReason();
-                addCause(VersionSelectionReasons.REJECTION.withReason(new RejectedByRuleReason(version, reason)));
-            } else if (rejectedVersion instanceof RejectedByAttributesVersion) {
-                addCause(VersionSelectionReasons.REJECTION.withReason(new RejectedByAttributesReason((RejectedByAttributesVersion) rejectedVersion)));
-            }
-        }
-    }
-
-    private void registerRejections(RejectedBySelectorVersion rejectedVersion, String version) {
-        if (rejectedBySelectors == null) {
-            rejectedBySelectors = LinkedHashMultimap.create();
-        }
-        rejectedBySelectors.put(rejectedVersion.getRejectionSelector(), version);
+    public VirtualPlatformState getPlatformState() {
+        return module.getPlatformState();
     }
 
     public void removeOutgoingEdges() {
@@ -431,112 +422,4 @@ public class ComponentState implements ComponentResolutionState, DependencyGraph
         }
         return null;
     }
-
-    private class RejectedBySelectorDescriptorBuilder implements Transformer<ComponentSelectionDescriptorInternal, ComponentSelectionDescriptorInternal> {
-        private final SelectorState selectorState;
-
-        public RejectedBySelectorDescriptorBuilder(SelectorState selectorState) {
-            this.selectorState = selectorState;
-        }
-
-        @Override
-        public ComponentSelectionDescriptorInternal transform(ComponentSelectionDescriptorInternal descriptor) {
-            if (rejectedBySelectors != null) {
-                Collection<String> rejectedByThisSelector = rejectedBySelectors.get(selectorState.getVersionConstraint().getRejectedSelector());
-                if (!rejectedByThisSelector.isEmpty()) {
-                    descriptor = descriptor.withReason(new RejectedBySelectorReason(rejectedByThisSelector, descriptor));
-                } else {
-                    rejectedByThisSelector = rejectedBySelectors.get(selectorState.getVersionConstraint().getRequiredSelector());
-                    if (!rejectedByThisSelector.isEmpty()) {
-                        descriptor = descriptor.withReason(new UnmatchedVersionsReason(rejectedByThisSelector, descriptor));
-                    }
-                }
-            }
-            return descriptor;
-        }
-    }
-
-    private static class RejectedByRuleReason implements Describable {
-        private final String version;
-        private final String reason;
-
-        private RejectedByRuleReason(String version, String reason) {
-            this.version = version;
-            this.reason = reason;
-        }
-
-        @Override
-        public String getDisplayName() {
-            return version + " by rule" + (reason != null ? " because " + reason : "");
-        }
-    }
-
-    private static class RejectedByAttributesReason implements Describable {
-        private final RejectedByAttributesVersion version;
-
-        private RejectedByAttributesReason(RejectedByAttributesVersion version) {
-            this.version = version;
-        }
-
-
-        @Override
-        public String getDisplayName() {
-            TreeFormatter formatter = new TreeFormatter();
-            version.describeTo(formatter);
-            return "version " + formatter;
-        }
-    }
-
-    private static class RejectedBySelectorReason implements Describable {
-
-        private final Collection<String> rejectedVersions;
-        private final ComponentSelectionDescriptorInternal descriptor;
-
-        private RejectedBySelectorReason(Collection<String> rejectedVersions, ComponentSelectionDescriptorInternal descriptor) {
-            this.rejectedVersions = rejectedVersions;
-            this.descriptor = descriptor;
-        }
-
-        @Override
-        public String getDisplayName() {
-            boolean hasCustomDescription = descriptor.hasCustomDescription();
-            StringBuilder sb = new StringBuilder(estimateSize(hasCustomDescription));
-            sb.append(rejectedVersions.size() > 1 ? "rejected versions " : "rejected version ");
-            Joiner.on(", ").appendTo(sb, rejectedVersions);
-            if (hasCustomDescription) {
-                sb.append(" because ").append(descriptor.getDescription());
-            }
-            return sb.toString();
-        }
-
-        private int estimateSize(boolean hasCustomDescription) {
-            return 20 + rejectedVersions.size() * 8 + (hasCustomDescription ? 24 : 0);
-        }
-    }
-
-    private static class UnmatchedVersionsReason implements Describable {
-        private final Collection<String> rejectedVersions;
-        private final ComponentSelectionDescriptorInternal descriptor;
-
-        private UnmatchedVersionsReason(Collection<String> rejectedVersions, ComponentSelectionDescriptorInternal descriptor) {
-            this.rejectedVersions = rejectedVersions;
-            this.descriptor = descriptor;
-        }
-
-        @Override
-        public String getDisplayName() {
-            boolean hasCustomDescription = descriptor.hasCustomDescription();
-            StringBuilder sb = new StringBuilder(estimateSize(hasCustomDescription));
-            sb.append(rejectedVersions.size() > 1 ? "didn't match versions " : "didn't match version ");
-            Joiner.on(", ").appendTo(sb, rejectedVersions);
-            if (hasCustomDescription) {
-                sb.append(" because ").append(descriptor.getDescription());
-            }
-            return sb.toString();
-        }
-
-        private int estimateSize(boolean hasCustomDescription) {
-            return 24 + rejectedVersions.size() * 8 + (hasCustomDescription ? 24 : 0);
-        }
-    }
 }
