@@ -23,6 +23,8 @@ import org.gradle.api.GradleException;
 import org.gradle.api.artifacts.Configuration;
 import org.gradle.api.artifacts.ModuleVersionIdentifier;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
+import org.gradle.api.artifacts.component.ComponentSelector;
+import org.gradle.api.artifacts.component.ModuleComponentSelector;
 import org.gradle.api.capabilities.Capability;
 import org.gradle.api.internal.artifacts.ComponentSelectorConverter;
 import org.gradle.api.internal.artifacts.ResolveContext;
@@ -325,26 +327,48 @@ public class DependencyGraphBuilder {
                 // We need to attach failures on unattached dependencies too, in case a node wasn't selected
                 // at all, but we still want to see an error message for it.
                 attachFailureToEdges(error, module.getUnattachedDependencies());
-            } else if (module.isVirtualPlatform() && module.hasCompetingForceSelectors()) {
+            } else if (module.isVirtualPlatform()) {
                 attachMultipleForceOnPlatformFailureToEdges(module);
             }
         }
     }
 
     private void attachMultipleForceOnPlatformFailureToEdges(ModuleResolveState module) {
-        List<EdgeState> forcedEdges = Lists.newArrayList();
+        List<EdgeState> forcedEdges = null;
+        boolean hasMultipleVersions = false;
+        String currentVersion = module.maybeFindForcedPlatformVersion();
         Set<ModuleResolveState> participatingModules = module.getPlatformState().getParticipatingModules();
         for (ModuleResolveState participatingModule : participatingModules) {
             for (EdgeState incomingEdge : participatingModule.getIncomingEdges()) {
-                if (incomingEdge.getSelector().isForce()) {
-                    // Filter out platform originating edges
-                    if (!incomingEdge.getFrom().getComponent().getModule().equals(module)) {
-                        forcedEdges.add(incomingEdge);
+                SelectorState selector = incomingEdge.getSelector();
+                if (isPlatformForcedEdge(selector)) {
+                    ComponentSelector componentSelector = selector.getSelector();
+                    if (componentSelector instanceof ModuleComponentSelector) {
+                        ModuleComponentSelector mcs = (ModuleComponentSelector) componentSelector;
+                        if (!incomingEdge.getFrom().getComponent().getModule().equals(module)) {
+                            if (forcedEdges == null) {
+                                forcedEdges = Lists.newArrayList();
+                            }
+                            forcedEdges.add(incomingEdge);
+                            if (currentVersion == null) {
+                                currentVersion = mcs.getVersion();
+                            } else {
+                                if (!currentVersion.equals(mcs.getVersion())) {
+                                    hasMultipleVersions = true;
+                                }
+                            }
+                        }
                     }
                 }
             }
         }
-        attachFailureToEdges(new GradleException("Multiple forces on different versions for virtual platform " + module.getId()), forcedEdges);
+        if (hasMultipleVersions) {
+            attachFailureToEdges(new GradleException("Multiple forces on different versions for virtual platform " + module.getId()), forcedEdges);
+        }
+    }
+
+    private static boolean isPlatformForcedEdge(SelectorState selector) {
+        return selector.isForce() && !selector.isSoftForce();
     }
 
     /**
@@ -372,7 +396,7 @@ public class DependencyGraphBuilder {
 
         // Visit the nodes prior to visiting the edges
         for (NodeState nodeState : resolveState.getNodes()) {
-            if (nodeState.isSelected()) {
+            if (nodeState.shouldIncludedInGraphResult()) {
                 visitor.visitNode(nodeState);
             }
         }
@@ -380,7 +404,7 @@ public class DependencyGraphBuilder {
         // Collect the components to sort in consumer-first order
         List<ComponentState> queue = Lists.newArrayListWithExpectedSize(resolveState.getNodeCount());
         for (ModuleResolveState module : resolveState.getModules()) {
-            if (module.getSelected() != null) {
+            if (module.getSelected() != null && !module.isVirtualPlatform()) {
                 queue.add(module.getSelected());
             }
         }
@@ -397,7 +421,7 @@ public class DependencyGraphBuilder {
                     }
                     for (EdgeState edge : node.getIncomingEdges()) {
                         ComponentState owner = edge.getFrom().getOwner();
-                        if (owner.getVisitState() == VisitState.NotSeen) {
+                        if (owner.getVisitState() == VisitState.NotSeen && !owner.getModule().isVirtualPlatform()) {
                             queue.add(pos, owner);
                             pos++;
                         } // else, already visited or currently visiting (which means a cycle), skip
