@@ -321,7 +321,7 @@ class NodeState implements DependencyGraphNode {
 
         DependencySubstitutionInternal details = substitutionResult.getResult();
         if (details != null && details.isUpdated()) {
-            return dependencyState.withTarget(details.getTarget(), details.getSelectionDescription());
+            return dependencyState.withTarget(details.getTarget(), details.getRuleDescriptors());
         }
         return dependencyState;
     }
@@ -375,6 +375,10 @@ class NodeState implements DependencyGraphNode {
         return !incomingEdges.isEmpty();
     }
 
+    public boolean shouldIncludedInGraphResult() {
+        return isSelected() && !component.getModule().isVirtualPlatform();
+    }
+
     private ModuleExclusion getModuleResolutionFilter(List<EdgeState> incomingEdges) {
         ModuleExclusions moduleExclusions = resolveState.getModuleExclusions();
         ModuleExclusion nodeExclusions = moduleExclusions.excludeAny(metaData.getExcludes());
