@@ -32,7 +32,7 @@ import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionP
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.strategy.VersionSelectorScheme;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionDescriptorInternal;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionReasonInternal;
-import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.VersionSelectionReasons;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.result.ComponentSelectionReasons;
 import org.gradle.api.tasks.diagnostics.internal.graph.nodes.DefaultSection;
 import org.gradle.api.tasks.diagnostics.internal.graph.nodes.DependencyEdge;
 import org.gradle.api.tasks.diagnostics.internal.graph.nodes.DependencyReportHeader;
@@ -170,7 +170,7 @@ public class DependencyInsightReporter {
             ComponentSelectionDescriptorInternal descriptor = (ComponentSelectionDescriptorInternal) entry;
             boolean hasCustomDescription = descriptor.hasCustomDescription();
 
-            if (VersionSelectionReasons.isCauseExpected(descriptor) && !hasCustomDescription) {
+            if (ComponentSelectionReasons.isCauseExpected(descriptor) && !hasCustomDescription) {
                 // Don't render empty 'requested' reason
                 continue;
             }
