@@ -17,6 +17,7 @@
 package org.gradle.ide.visualstudio.internal;
 
 import org.gradle.api.Action;
+import org.gradle.api.internal.CollectionCallbackActionDecorator;
 import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.model.ObjectFactory;
 import org.gradle.ide.visualstudio.VisualStudioRootExtension;
@@ -27,8 +28,8 @@ import org.gradle.plugins.ide.internal.IdeArtifactRegistry;
 public class DefaultVisualStudioRootExtension extends DefaultVisualStudioExtension implements VisualStudioRootExtension, VisualStudioExtensionInternal {
     private final VisualStudioSolution solution;
 
-    public DefaultVisualStudioRootExtension(String projectName, Instantiator instantiator, ObjectFactory objectFactory, FileResolver fileResolver, IdeArtifactRegistry ideArtifactRegistry) {
-        super(instantiator, fileResolver, ideArtifactRegistry);
+    public DefaultVisualStudioRootExtension(String projectName, Instantiator instantiator, ObjectFactory objectFactory, FileResolver fileResolver, IdeArtifactRegistry ideArtifactRegistry, CollectionCallbackActionDecorator collectionCallbackActionDecorator) {
+        super(instantiator, fileResolver, ideArtifactRegistry, collectionCallbackActionDecorator);
         this.solution = objectFactory.newInstance(DefaultVisualStudioSolution.class, projectName);
     }
 
