@@ -28,7 +28,9 @@ import org.gradle.api.artifacts.dsl.DependencyHandler;
 import org.gradle.api.artifacts.result.ResolvedArtifactResult;
 import org.gradle.api.artifacts.result.UnresolvedDependencyResult;
 import org.gradle.api.internal.artifacts.DefaultModuleVersionIdentifier;
+import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.api.internal.project.ProjectStateRegistry;
+import org.gradle.internal.Factory;
 import org.gradle.plugins.ide.idea.model.Dependency;
 import org.gradle.plugins.ide.idea.model.FilePath;
 import org.gradle.plugins.ide.idea.model.IdeaModule;
@@ -39,6 +41,7 @@ import org.gradle.plugins.ide.internal.resolver.IdeDependencySet;
 import org.gradle.plugins.ide.internal.resolver.IdeDependencyVisitor;
 import org.gradle.plugins.ide.internal.resolver.UnresolvedIdeDependencyHandler;
 
+import javax.annotation.Nullable;
 import java.io.File;
 import java.util.Collection;
 import java.util.Collections;
@@ -97,13 +100,21 @@ public class IdeaDependenciesProvider {
     }
 
     private IdeaDependenciesVisitor visitDependencies(IdeaModule ideaModule, GeneratedIdeaScope scope) {
-        DependencyHandler handler = ideaModule.getProject().getDependencies();
-        Collection<Configuration> plusConfigurations = getPlusConfigurations(ideaModule, scope);
-        Collection<Configuration> minusConfigurations = getMinusConfigurations(ideaModule, scope);
+        ProjectInternal projectInternal = (ProjectInternal) ideaModule.getProject();
+        final DependencyHandler handler = projectInternal.getDependencies();
+        final Collection<Configuration> plusConfigurations = getPlusConfigurations(ideaModule, scope);
+        final Collection<Configuration> minusConfigurations = getMinusConfigurations(ideaModule, scope);
+
+        final IdeaDependenciesVisitor visitor = new IdeaDependenciesVisitor(ideaModule, scope.name());
+        return projectInternal.getMutationState().withMutableState(new Factory<IdeaDependenciesVisitor>() {
+            @Nullable
+            @Override
+            public IdeaDependenciesVisitor create() {
+                new IdeDependencySet(handler, plusConfigurations, minusConfigurations).visit(visitor);
+                return visitor;
+            }
+        });
 
-        IdeaDependenciesVisitor visitor = new IdeaDependenciesVisitor(ideaModule, scope.name());
-        new IdeDependencySet(handler, plusConfigurations, minusConfigurations).visit(visitor);
-        return visitor;
     }
 
     private Collection<Configuration> getPlusConfigurations(IdeaModule ideaModule, GeneratedIdeaScope scope) {
@@ -202,7 +213,7 @@ public class IdeaDependenciesProvider {
          */
         @Override
         public void visitUnresolvedDependency(UnresolvedDependencyResult unresolvedDependency) {
-            File unresolvedFile = unresolvedIdeDependencyHandler.asFile(unresolvedDependency);
+            File unresolvedFile = unresolvedIdeDependencyHandler.asFile(unresolvedDependency, ideaModule.getContentRoot());
             fileDependencies.add(new SingleEntryModuleLibrary(toPath(ideaModule, unresolvedFile), scope));
             unresolvedDependencies.put(unresolvedDependency.getAttempted(), unresolvedDependency);
         }
