@@ -101,7 +101,7 @@ public class DefaultGradleDistribution implements GradleDistribution {
             return javaVersion.compareTo(JavaVersion.VERSION_1_7) >= 0 && javaVersion.compareTo(JavaVersion.VERSION_1_10) <= 0;
         }
 
-        return javaVersion.compareTo(JavaVersion.VERSION_1_8) >= 0 && javaVersion.compareTo(JavaVersion.VERSION_1_10) <= 0;
+        return javaVersion.compareTo(JavaVersion.VERSION_1_8) >= 0;
     }
 
     public boolean worksWith(OperatingSystem os) {
@@ -127,22 +127,6 @@ public class DefaultGradleDistribution implements GradleDistribution {
         return worksWith(javaVersion);
     }
 
-    public boolean isToolingApiNonAsciiOutputSupported() {
-        if (OperatingSystem.current().isWindows()) {
-            return !isVersion("1.0-milestone-7") && !isVersion("1.0-milestone-8") && !isVersion("1.0-milestone-8a");
-        }
-        return true;
-    }
-
-    public boolean isToolingApiDaemonBaseDirSupported() {
-        return isSameOrNewer("2.2-rc-1");
-    }
-
-    @Override
-    public boolean isToolingApiEventsInEmbeddedModeSupported() {
-        return isSameOrNewer("2.6-rc-1");
-    }
-
     @Override
     public boolean isToolingApiLocksBuildActionClasses() {
         return isSameOrOlder("3.0");
@@ -206,6 +190,59 @@ public class DefaultGradleDistribution implements GradleDistribution {
         return isSameOrNewer("5.0");
     }
 
+    @Override
+    public boolean isToolingApiRetainsOriginalFailureOnCancel() {
+        // Versions before 5.1 would unpack the exception and throw part of it, losing some context
+        return isSameOrNewer("5.1");
+    }
+
+    @Override
+    public boolean isToolingApiDoesNotAddCausesOnTaskCancel() {
+        // Versions before 5.1 would sometimes add some additional 'build cancelled' exceptions
+        return isSameOrNewer("5.1");
+    }
+
+    @Override
+    public boolean isToolingApiHasCauseOnCancel() {
+        // Versions before 3.2 would throw away the cause. There was also a regression in 4.0.x
+        return isSameOrNewer("3.2") && !(isSameOrNewer("4.0") && isSameOrOlder("4.0.2"));
+    }
+
+    @Override
+    public boolean isToolingApiHasCauseOnForcedCancel() {
+        // Versions before 5.1 would discard context on forced cancel
+        return isSameOrNewer("5.1");
+    }
+
+    @Override
+    public boolean isToolingApiLogsFailureOnCancel() {
+        // Versions before 4.1 would log "CONFIGURE SUCCESSFUL" for model/action execution (but "BUILD FAILED" for task/test execution)
+        return isSameOrNewer("4.1");
+    }
+
+    @Override
+    public boolean isToolingApiHasCauseOnPhasedActionFail() {
+        return isSameOrNewer("5.1");
+    }
+
+    @Override
+    public boolean isToolingApiMergesStderrIntoStdout() {
+        return isSameOrNewer("4.7") && isSameOrOlder("5.0");
+    }
+
+    @Override
+    public boolean isToolingApiLogsConfigureSummary() {
+        return isSameOrNewer("2.14");
+    }
+
+    @Override
+    public <T> T selectOutputWithFailureLogging(T stdout, T stderr) {
+        if (isSameOrNewer("4.0") && isSameOrOlder("4.6") || isSameOrNewer("5.1")) {
+            return stderr;
+        }
+        return stdout;
+    }
+
     protected boolean isSameOrNewer(String otherVersion) {
         return isVersion(otherVersion) || version.compareTo(GradleVersion.version(otherVersion)) > 0;
     }
