@@ -20,10 +20,8 @@ import com.google.common.base.Joiner;
 import com.google.common.collect.Lists;
 import com.google.common.collect.Sets;
 import com.googlecode.jatl.Html;
+import groovy.json.JsonOutput;
 import org.apache.commons.lang.StringUtils;
-import org.gradle.api.Transformer;
-import org.gradle.performance.measure.DataSeries;
-import org.gradle.performance.measure.Duration;
 
 import javax.annotation.Nullable;
 import java.io.IOException;
@@ -34,6 +32,12 @@ import java.util.List;
 import java.util.Set;
 
 public class TestPageGenerator extends HtmlPageGenerator<PerformanceTestHistory> {
+    private final String projectName;
+
+    public TestPageGenerator(String projectName) {
+        this.projectName = projectName;
+    }
+
     @Override
     protected int getDepth() {
         return 1;
@@ -42,6 +46,7 @@ public class TestPageGenerator extends HtmlPageGenerator<PerformanceTestHistory>
     @Override
     public void render(final PerformanceTestHistory testHistory, Writer writer) throws IOException {
         // TODO: Add test name to the report
+        // @formatter:off
         new MetricsHtml(writer) {{
             html();
             head();
@@ -55,10 +60,7 @@ public class TestPageGenerator extends HtmlPageGenerator<PerformanceTestHistory>
             p().text("Tasks: " + getTasks(testHistory)).end();
             p().text("Clean tasks: " + getCleanTasks(testHistory)).end();
 
-            addPerformanceGraph("Average total time", "totalTimeChart", "totalTime", "total time", "s");
-
-            div().id("tooltip").end();
-            div().id("controls").end();
+            addPerformanceGraphs();
 
             h3().text("Test details").end();
             table().classAttr("test-details");
@@ -99,6 +101,8 @@ public class TestPageGenerator extends HtmlPageGenerator<PerformanceTestHistory>
             for (String label : testHistory.getScenarioLabels()) {
                 renderHeaderForSamples(label);
             }
+            th().text("Confidence").end();
+            th().text("Difference").end();
             th().text("Test version").end();
             th().text("Operating System").end();
             th().text("Host").end();
@@ -115,19 +119,14 @@ public class TestPageGenerator extends HtmlPageGenerator<PerformanceTestHistory>
                 PerformanceTestExecution results = testHistory.getExecutions().get(i);
                 tr();
                 id("result" + results.getExecutionId());
-                textCell(format.timestamp(new Date(results.getStartTime())));
+                renderDateAndLink(results);
                 textCell(results.getVcsBranch());
 
                 td();
                 renderVcsLinks(results, findPreviousExecutionInSameBranch(results, testHistory, i));
                 end();
 
-                final List<MeasuredOperationList> scenarios = results.getScenarios();
-                renderSamplesForExperiment(scenarios, new Transformer<DataSeries<Duration>, MeasuredOperationList>() {
-                    public DataSeries<Duration> transform(MeasuredOperationList original) {
-                        return original.getTotalTime();
-                    }
-                });
+                renderSamplesForExperiment(results.getScenarios());
                 textCell(results.getVersionUnderTest());
                 textCell(results.getOperatingSystem());
                 textCell(results.getHost());
@@ -146,6 +145,17 @@ public class TestPageGenerator extends HtmlPageGenerator<PerformanceTestHistory>
             endAll();
         }
 
+            private void renderDateAndLink(PerformanceTestExecution results) {
+                td();
+                    String date = FormatSupport.timestamp(new Date(results.getStartTime()));
+                    if (results.getTeamCityBuildId() == null) {
+                        text(date);
+                    } else {
+                        a().href("https://builds.gradle.org/viewLog.html?buildId=" + results.getTeamCityBuildId()).target("_blank").text(date).end();
+                    }
+                end();
+            }
+
             private void renderVcsLinks(PerformanceTestExecution results, PerformanceTestExecution previousResults) {
                 List<GitHubLink> vcsCommits = createGitHubLinks(results.getVcsCommits());
                 for (int i = 0; i < vcsCommits.size(); i++) {
@@ -176,17 +186,58 @@ public class TestPageGenerator extends HtmlPageGenerator<PerformanceTestHistory>
                 return previousResults;
             }
 
-            private void addPerformanceGraph(String heading, String chartId, String jsonFieldName, String fieldLabel, String fieldUnit) {
-                h3().text(heading).end();
-                div().id(chartId).classAttr("chart");
-                p().text("Loading...").end();
+            private void addPerformanceGraphs() {
+                List<Chart> charts = Lists.newArrayList(new Chart("totalTime", "total time", "s", "totalTimeChart"));
+                if(testHistory instanceof CrossVersionPerformanceTestHistory) {
+                    charts.add(new Chart("confidence", "confidence", "%", "confidenceChart"));
+                    charts.add(new Chart("difference", "difference", "%", "differenceChart"));
+                }
+
+                charts.forEach(chart -> {
+                    h3().text(StringUtils.capitalize(chart.getLabel()) + " (" + chart.getUnit() + ")").end();
+                    div().classAttr("chart").id(chart.getChartId());
+                        p().text("Loading...").end();
+                    end();
+                    div().id(chart.getChartId() + "Legend").end();
+                });
+
                 script();
-                text("performanceTests.createPerformanceGraph('" + urlEncode(testHistory.getId()) + ".json', function(data) { return data." + jsonFieldName + "}, '" + fieldLabel + "', '" + fieldUnit + "', '" + chartId + "');");
-                end();
+                    raw("performanceTests.createPerformanceGraph('" + urlEncode(testHistory.getId()) + ".json', " + JsonOutput.toJson(charts) + ")");
                 end();
             }
         };
     }
+    // @formatter:on
+
+    private static class Chart {
+        private String field;
+        private String label;
+        private String unit;
+        private String chartId;
+
+        private Chart(String field, String label, String unit, String chartId) {
+            this.field = field;
+            this.label = label;
+            this.unit = unit;
+            this.chartId = chartId;
+        }
+
+        public String getField() {
+            return field;
+        }
+
+        public String getLabel() {
+            return label;
+        }
+
+        public String getUnit() {
+            return unit;
+        }
+
+        public String getChartId() {
+            return chartId;
+        }
+    }
 
     private static String getTasks(PerformanceTestHistory testHistory) {
         PerformanceTestExecution performanceTestExecution = getExecution(testHistory);
@@ -220,13 +271,12 @@ public class TestPageGenerator extends HtmlPageGenerator<PerformanceTestHistory>
             cleanTasks.add("clean" + StringUtils.capitalize(scenario.getTestProject()));
         }
 
-        return "To reproduce, run ./gradlew "
-            + Joiner.on(' ').join(cleanTasks)
-            + " "
-            + Joiner.on(' ').join(templates)
-            + " cleanPerformanceAdhocTest performanceAdhocTest --scenarios "
-            + "'" + history.getDisplayName() + "'"
-            + " -x prepareSamples";
+        return String.format("To reproduce, run ./gradlew %s %s cleanPerformanceAdhocTest :%s:performanceAdhocTest --scenarios '%s' -x prepareSamples",
+            Joiner.on(' ').join(cleanTasks),
+            Joiner.on(' ').join(templates),
+            projectName,
+            history.getDisplayName()
+        );
     }
 
     private static class GitHubLink {
