@@ -27,7 +27,7 @@ import org.gradle.api.provider.SetProperty;
 import org.gradle.language.BinaryCollection;
 import org.gradle.language.ComponentWithBinaries;
 import org.gradle.language.ComponentWithDependencies;
-import org.gradle.nativeplatform.OperatingSystemFamily;
+import org.gradle.nativeplatform.TargetMachine;
 
 /**
  * Configuration for a C++ component, such as a library or executable, defining the source files and private header directories that make up the component. Private headers are those that are visible only to the source files of the component.
@@ -97,9 +97,14 @@ public interface CppComponent extends ComponentWithBinaries, ComponentWithDepend
     BinaryCollection<? extends CppBinary> getBinaries();
 
     /**
-     * Specifies the operating systems for the target machine.
+     * Specifies the target machines this component should be built for.  The "machines" extension property (see {@link org.gradle.nativeplatform.TargetMachineFactory}) can be used to construct common operating system and architecture combinations.
      *
-     * @since 4.7
+     * <p>For example:</p>
+     * <pre>
+     * targetMachines = [machines.linux.x86_64, machines.windows.x86_64]
+     * </pre>
+     *
+     * @since 5.1
      */
-    SetProperty<OperatingSystemFamily> getOperatingSystems();
+    SetProperty<TargetMachine> getTargetMachines();
 }
