@@ -17,11 +17,21 @@
 package org.gradle.language.cpp.internal;
 
 import org.gradle.language.cpp.CppPlatform;
-import org.gradle.nativeplatform.platform.internal.DefaultNativePlatform;
+import org.gradle.nativeplatform.OperatingSystemFamily;
+import org.gradle.nativeplatform.TargetMachine;
+import org.gradle.nativeplatform.platform.internal.ImmutableDefaultNativePlatform;
 import org.gradle.nativeplatform.platform.internal.NativePlatformInternal;
 
-public class DefaultCppPlatform extends DefaultNativePlatform implements CppPlatform {
-    public DefaultCppPlatform(String name, NativePlatformInternal targetMachine) {
-        super(name, targetMachine.getOperatingSystem(), targetMachine.getArchitecture());
+public class DefaultCppPlatform extends ImmutableDefaultNativePlatform implements CppPlatform {
+    private final OperatingSystemFamily operatingSystemFamily;
+
+    public DefaultCppPlatform(String name, TargetMachine targetMachine, NativePlatformInternal targetNativePlatform) {
+        super(name, targetNativePlatform.getOperatingSystem(), targetNativePlatform.getArchitecture());
+        this.operatingSystemFamily = targetMachine.getOperatingSystemFamily();
+    }
+
+    @Override
+    public OperatingSystemFamily getOperatingSystemFamily() {
+        return operatingSystemFamily;
     }
 }
