@@ -52,6 +52,7 @@ import org.gradle.internal.nativeintegration.console.ConsoleMetaData;
 import org.gradle.internal.nativeintegration.console.FallbackConsoleMetaData;
 import org.gradle.internal.time.Clock;
 
+import javax.annotation.Nullable;
 import java.io.OutputStream;
 import java.io.OutputStreamWriter;
 import java.util.concurrent.atomic.AtomicReference;
@@ -150,31 +151,31 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
 
     @Override
     public void attachConsole(OutputStream outputStream, OutputStream errorStream, ConsoleOutput consoleOutput) {
-        attachConsole(outputStream, errorStream, consoleOutput, FallbackConsoleMetaData.INSTANCE);
+        attachConsole(outputStream, errorStream, consoleOutput, null);
     }
 
     @Override
-    public void attachConsole(OutputStream outputStream, OutputStream errorStream, ConsoleOutput consoleOutput, ConsoleMetaData consoleMetadata) {
+    public void attachConsole(OutputStream outputStream, OutputStream errorStream, ConsoleOutput consoleOutput, @Nullable ConsoleMetaData consoleMetadata) {
         synchronized (lock) {
+            if (consoleMetadata == null) {
+                consoleMetadata = consoleOutput == ConsoleOutput.Plain ? FallbackConsoleMetaData.NOT_ATTACHED : FallbackConsoleMetaData.ATTACHED;
+            }
             StandardOutputListener outputListener = new StreamBackedStandardOutputListener(outputStream);
             StandardOutputListener errorListener = new StreamBackedStandardOutputListener(errorStream);
             if (consoleOutput == ConsoleOutput.Plain) {
-                addPlainConsole(outputListener, errorListener, consoleMetadata != null && (consoleMetadata.isStdErr() && consoleMetadata.isStdOut()));
+                addPlainConsole(outputListener, errorListener, consoleMetadata.isStdOut() && consoleMetadata.isStdErr());
             } else {
-                if (consoleMetadata == null) {
-                    consoleMetadata = FallbackConsoleMetaData.INSTANCE;
-                }
                 Console console;
                 if (consoleMetadata.isStdOut()) {
                     OutputStreamWriter writer = new OutputStreamWriter(outputStream);
-                    console =new AnsiConsole(writer, writer, getColourMap(), consoleMetadata, true);
+                    console = new AnsiConsole(writer, writer, getColourMap(), consoleMetadata, true);
                 } else if (consoleMetadata.isStdErr()) {
                     OutputStreamWriter writer = new OutputStreamWriter(errorStream);
-                    console =new AnsiConsole(writer, writer, getColourMap(), consoleMetadata, true);
+                    console = new AnsiConsole(writer, writer, getColourMap(), consoleMetadata, true);
                 } else {
                     console = null;
                 }
-                addRichConsole(console, consoleMetadata.isStdOut(), consoleMetadata.isStdErr(), outputListener, errorListener, consoleMetadata, consoleOutput == ConsoleOutput.Verbose);
+                addRichConsole(console, outputListener, errorListener, consoleMetadata, consoleOutput == ConsoleOutput.Verbose);
             }
         }
     }
@@ -246,16 +247,18 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
         }
     }
 
-    public OutputEventRenderer addRichConsole(Console console, boolean stdoutAttachedToConsole, boolean stderrAttachedToConsole, ConsoleMetaData consoleMetaData) {
-        return addRichConsole(console, stdoutAttachedToConsole, stderrAttachedToConsole, consoleMetaData, false);
+    public OutputEventRenderer addRichConsole(Console console, ConsoleMetaData consoleMetaData) {
+        return addRichConsole(console, consoleMetaData, false);
     }
 
-    public OutputEventRenderer addRichConsole(Console console, boolean stdoutAttachedToConsole, boolean stderrAttachedToConsole, ConsoleMetaData consoleMetaData, boolean verbose) {
-        return addRichConsole(console, stdoutAttachedToConsole, stderrAttachedToConsole, new StreamBackedStandardOutputListener((Appendable) originalStdOut), new StreamBackedStandardOutputListener((Appendable)originalStdErr), consoleMetaData, verbose);
+    public OutputEventRenderer addRichConsole(Console console, ConsoleMetaData consoleMetaData, boolean verbose) {
+        return addRichConsole(console, new StreamBackedStandardOutputListener((Appendable) originalStdOut), new StreamBackedStandardOutputListener((Appendable) originalStdErr), consoleMetaData, verbose);
     }
 
-    private OutputEventRenderer addRichConsole(Console console, boolean stdoutAttachedToConsole, boolean stderrAttachedToConsole, StandardOutputListener outputListener, StandardOutputListener errorListener, ConsoleMetaData consoleMetaData, boolean verbose) {
+    private OutputEventRenderer addRichConsole(Console console, StandardOutputListener outputListener, StandardOutputListener errorListener, ConsoleMetaData consoleMetaData, boolean verbose) {
         OutputEventListener consoleChain;
+        boolean stdoutAttachedToConsole = consoleMetaData.isStdOut();
+        boolean stderrAttachedToConsole = consoleMetaData.isStdErr();
         if (stdoutAttachedToConsole && stderrAttachedToConsole) {
             OutputEventListener consoleListener = new StyledTextOutputBackedRenderer(console.getBuildOutputArea());
             consoleChain = getRichConsoleChain(console, consoleMetaData, verbose, consoleListener);
@@ -300,7 +303,7 @@ public class OutputEventRenderer implements OutputEventListener, LoggingRouter {
     }
 
     private OutputEventListener getPlainConsoleChain(boolean redirectStderr) {
-        return getPlainConsoleChain(new StreamBackedStandardOutputListener((Appendable) originalStdOut), new StreamBackedStandardOutputListener((Appendable)originalStdErr), redirectStderr, true);
+        return getPlainConsoleChain(new StreamBackedStandardOutputListener((Appendable) originalStdOut), new StreamBackedStandardOutputListener((Appendable) originalStdErr), redirectStderr, true);
     }
 
     private OutputEventListener getPlainConsoleChain(StandardOutputListener outputListener, StandardOutputListener errorListener, boolean redirectStderr, boolean verbose) {
