@@ -46,12 +46,21 @@ abstract class VisualCppCompilerArgsTransformer<T extends NativeCompileSpec> imp
         args.add(getLanguageOption());
         args.add("/nologo");
         args.add("/c");
+        if (spec.isDebuggable()) {
+            args.add("/Zi");
+        }
+        if (spec.isOptimized()) {
+            args.add("/O2");
+        }
     }
 
     protected void addIncludeArgs(T spec, List<String> args) {
         for (File file : spec.getIncludeRoots()) {
             args.add("/I" + file.getAbsolutePath());
         }
+        for (File file : spec.getSystemIncludeRoots()) {
+            args.add("/I" + file.getAbsolutePath());
+        }
     }
 
     protected void addMacroArgs(T spec, List<String> args) {
