@@ -17,13 +17,15 @@
 package org.gradle.play.tasks;
 
 import org.gradle.api.Incubating;
+import org.gradle.api.file.DirectoryProperty;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.internal.ConventionTask;
 import org.gradle.api.tasks.Classpath;
-import org.gradle.api.tasks.InputFile;
 import org.gradle.api.tasks.InputFiles;
 import org.gradle.api.tasks.Internal;
 import org.gradle.api.tasks.Nested;
+import org.gradle.api.tasks.PathSensitive;
+import org.gradle.api.tasks.PathSensitivity;
 import org.gradle.api.tasks.TaskAction;
 import org.gradle.api.tasks.compile.BaseForkOptions;
 import org.gradle.deployment.internal.DeploymentRegistry;
@@ -49,19 +51,16 @@ public class PlayRun extends ConventionTask {
 
     private int httpPort;
 
-    @InputFile
+    private final DirectoryProperty workingDir = getProject().getObjects().directoryProperty();
+
     private File applicationJar;
 
-    @InputFile
     private File assetsJar;
 
-    @InputFiles
     private Set<File> assetsDirs;
 
-    @Classpath
     private FileCollection runtimeClasspath;
 
-    @Classpath
     private FileCollection changingClasspath;
 
     private BaseForkOptions forkOptions;
@@ -86,7 +85,7 @@ public class PlayRun extends ConventionTask {
         PlayApplicationDeploymentHandle deploymentHandle = deploymentRegistry.get(deploymentId, PlayApplicationDeploymentHandle.class);
 
         if (deploymentHandle == null) {
-            PlayRunSpec spec = new DefaultPlayRunSpec(runtimeClasspath, changingClasspath, applicationJar, assetsJar, assetsDirs, getProject().getProjectDir(), getForkOptions(), getHttpPort());
+            PlayRunSpec spec = new DefaultPlayRunSpec(runtimeClasspath, changingClasspath, applicationJar, assetsJar, assetsDirs, workingDir.get().getAsFile(), getForkOptions(), getHttpPort());
             PlayApplicationRunner playApplicationRunner = playToolProvider.get(PlayApplicationRunner.class);
             deploymentHandle = deploymentRegistry.start(deploymentId, DeploymentRegistry.ChangeBehavior.BLOCK, PlayApplicationDeploymentHandle.class, spec, playApplicationRunner);
 
@@ -112,9 +111,20 @@ public class PlayRun extends ConventionTask {
         this.httpPort = httpPort;
     }
 
+    /**
+     * The working directory.
+     *
+     * @since 4.4
+     */
+    @Internal
+    public DirectoryProperty getWorkingDir() {
+        return workingDir;
+    }
+
     /**
      * The Play application jar to run.
      */
+    @Classpath
     public File getApplicationJar() {
         return applicationJar;
     }
@@ -126,6 +136,7 @@ public class PlayRun extends ConventionTask {
     /**
      * The assets jar to run with the Play application.
      */
+    @Classpath
     public File getAssetsJar() {
         return assetsJar;
     }
@@ -137,6 +148,8 @@ public class PlayRun extends ConventionTask {
     /**
      * The directories of the assets for the Play application (for live reload functionality).
      */
+    @InputFiles
+    @PathSensitive(PathSensitivity.RELATIVE)
     public Set<File> getAssetsDirs() {
         return assetsDirs;
     }
@@ -145,10 +158,30 @@ public class PlayRun extends ConventionTask {
         this.assetsDirs = assetsDirs;
     }
 
+    /**
+     * The runtime classpath for the Play application.
+     *
+     * @since 5.0
+     */
+    @Classpath
+    public FileCollection getRuntimeClasspath() {
+        return runtimeClasspath;
+    }
+
     public void setRuntimeClasspath(FileCollection runtimeClasspath) {
         this.runtimeClasspath = runtimeClasspath;
     }
 
+    /**
+     * The changing classpath for the Play application.
+     *
+     * @since 5.0
+     */
+    @Classpath
+    public FileCollection getChangingClasspath() {
+        return changingClasspath;
+    }
+
     public void setChangingClasspath(FileCollection changingClasspath) {
         this.changingClasspath = changingClasspath;
     }
